@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2002 Patrick Julien <freak@codepimps.org>
+ *  SPDX-FileCopyrightText: 2002 Patrick Julien <freak@codepimps.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 #ifndef KIS_CONFIG_H_
 #define KIS_CONFIG_H_
@@ -22,6 +10,7 @@
 #include <QStringList>
 #include <QList>
 #include <QColor>
+#include <QObject>
 
 #include <ksharedconfig.h>
 #include <kconfiggroup.h>
@@ -35,6 +24,7 @@ class KoColorSpace;
 class KisSnapConfig;
 class QSettings;
 class KisOcioConfiguration;
+struct KisCumulativeUndoData;
 
 class KRITAUI_EXPORT KisConfig
 {
@@ -48,12 +38,15 @@ public:
 
     ~KisConfig();
 
+public Q_SLOTS:
+    /// Log the most interesting settings to the usage log
+    void logImportantSettings() const;
+public:
+
     bool disableTouchOnCanvas(bool defaultValue = false) const;
     void setDisableTouchOnCanvas(bool value) const;
 
-    bool disableTouchRotation(bool defaultValue = false) const;
-    void setDisableTouchRotation(bool value) const;
-
+    // XXX Unused?
     bool useProjections(bool defaultValue = false) const;
     void setUseProjections(bool useProj) const;
 
@@ -66,14 +59,8 @@ public:
     bool useCumulativeUndoRedo(bool defaultValue = false) const;
     void setCumulativeUndoRedo(bool value);
 
-    double stackT1(bool defaultValue = false) const;
-    void setStackT1(int T1);
-
-    double stackT2(bool defaultValue = false) const;
-    void setStackT2(int T2);
-
-    int stackN(bool defaultValue = false) const;
-    void setStackN(int N);
+    KisCumulativeUndoData cumulativeUndoData(bool defaultValue = false) const;
+    void setCumulativeUndoData(KisCumulativeUndoData value);
 
     qint32 defImageWidth(bool defaultValue = false) const;
     void defImageWidth(qint32 width) const;
@@ -87,6 +74,8 @@ public:
     int preferredVectorImportResolutionPPI(bool defaultValue = false) const;
     void setPreferredVectorImportResolutionPPI(int value) const;
 
+    bool useDefaultColorSpace(bool defaultvalue = false) const;
+    void setUseDefaultColorSpace(bool value) const;
     /**
      * @return the id of the default color model used for creating new images.
      */
@@ -123,6 +112,21 @@ public:
     OutlineStyle newOutlineStyle(bool defaultValue = false) const;
     void setNewOutlineStyle(OutlineStyle style);
 
+    OutlineStyle lastUsedOutlineStyle(bool defaultValue = false) const;
+    void setLastUsedOutlineStyle(OutlineStyle style);
+
+    bool separateEraserCursor(bool defaultValue = false) const;
+    void setSeparateEraserCursor(bool value) const;
+
+    CursorStyle eraserCursorStyle(bool defaultValue = false) const;
+    void setEraserCursorStyle(CursorStyle style);
+
+    QColor getEraserCursorMainColor(bool defaultValue = false) const;
+    void setEraserCursorMainColor(const QColor& v) const;
+
+    OutlineStyle eraserOutlineStyle(bool defaultValue = false) const;
+    void setEraserOutlineStyle(OutlineStyle style);
+
     QRect colorPreviewRect() const;
     void setColorPreviewRect(const QRect &rect);
 
@@ -138,6 +142,8 @@ public:
     /// the custom monitor profile set by the user, depending on the configuration
     const KoColorProfile *displayProfile(int screen) const;
 
+    const QString getScreenStringIdentfier(int screenNo) const;
+
     QString workingColorSpace(bool defaultValue = false) const;
     void setWorkingColorSpace(const QString & workingColorSpace) const;
 
@@ -177,6 +183,9 @@ public:
     qint32 pasteBehaviour(bool defaultValue = false) const;
     void setPasteBehaviour(qint32 behaviour) const;
 
+    qint32 pasteFormat(bool defaultValue) const;
+    void setPasteFormat(qint32 format);
+
     qint32 monitorRenderIntent(bool defaultValue = false) const;
     void setRenderIntent(qint32 monitorRenderIntent) const;
 
@@ -186,14 +195,12 @@ public:
     int openGLFilteringMode(bool defaultValue = false) const;
     void setOpenGLFilteringMode(int filteringMode);
 
+    void setWidgetStyle(QString name);
+    QString widgetStyle(bool defaultValue = false);
+
     bool useOpenGLTextureBuffer(bool defaultValue = false) const;
     void setUseOpenGLTextureBuffer(bool useBuffer);
 
-    bool disableVSync(bool defaultValue = false) const;
-    void setDisableVSync(bool disableVSync);
-
-    bool showAdvancedOpenGLSettings(bool defaultValue = false) const;
-
     bool forceOpenGLFenceWorkaround(bool defaultValue = false) const;
 
     int numMipmapLevels(bool defaultValue = false) const;
@@ -233,7 +240,7 @@ public:
     void setCheckSize(qint32 checkSize) const;
 
     bool scrollCheckers(bool defaultValue = false) const;
-    void setScrollingCheckers(bool scollCheckers) const;
+    void setScrollingCheckers(bool scrollCheckers) const;
 
     QColor checkersColor1(bool defaultValue = false) const;
     void setCheckersColor1(const QColor & v) const;
@@ -247,6 +254,9 @@ public:
     bool hideScrollbars(bool defaultValue = false) const;
     void setHideScrollbars(bool value) const;
 
+    bool scrollbarZoomEnabled(bool defaultValue = false) const;
+    void setScrollbarZoomEnabled(bool enabled) const;
+
     bool antialiasCurves(bool defaultValue = false) const;
     void setAntialiasCurves(bool v) const;
 
@@ -265,6 +275,12 @@ public:
     bool forceAlwaysFullSizedOutline(bool defaultValue = false) const;
     void setForceAlwaysFullSizedOutline(bool value) const;
 
+    bool showEraserOutlineWhilePainting(bool defaultValue = false) const;
+    void setShowEraserOutlineWhilePainting(bool showEraserOutlineWhilePainting) const;
+
+    bool forceAlwaysFullSizedEraserOutline(bool defaultValue = false) const;
+    void setForceAlwaysFullSizedEraserOutline(bool value) const;
+
     enum SessionOnStartup {
         SOS_BlankSession,
         SOS_PreviousSession,
@@ -276,6 +292,9 @@ public:
     bool saveSessionOnQuit(bool defaultValue) const;
     void setSaveSessionOnQuit(bool value);
 
+    bool hideDevFundBanner(bool defaultValue = false) const;
+    void setHideDevFundBanner(bool value = true);
+
     qreal outlineSizeMinimum(bool defaultValue = false) const;
     void setOutlineSizeMinimum(qreal outlineSizeMinimum) const;
 
@@ -341,6 +360,9 @@ public:
     bool hideDockersFullscreen(bool defaultValue = false) const;
     void setHideDockersFullscreen(const bool value) const;
 
+    bool showDockerTitleBars(bool defaultValue = false) const;
+    void setShowDockerTitleBars(const bool value) const;
+
     bool showDockers(bool defaultValue = false) const;
     void setShowDockers(const bool value) const;
 
@@ -415,6 +437,9 @@ public:
     int layerThumbnailSize(bool defaultValue = false) const;
     void setLayerThumbnailSize(int size);
 
+    int layerTreeIndentation(bool defaultValue = false) const;
+    void setLayerTreeIndentation(int percentage);
+
 
     bool sliderLabels(bool defaultValue = false) const;
     void setSliderLabels(bool enabled);
@@ -432,7 +457,7 @@ public:
     void setShowSingleChannelAsColor(bool asColor);
 
     bool hidePopups(bool defaultValue = false) const;
-    void setHidePopups(bool hidepopups);
+    void setHidePopups(bool hidePopups);
 
     int numDefaultLayers(bool defaultValue = false) const;
     void setNumDefaultLayers(int num);
@@ -441,7 +466,7 @@ public:
     void setDefaultBackgroundOpacity(quint8 value);
 
     QColor defaultBackgroundColor(bool defaultValue = false) const;
-    void setDefaultBackgroundColor(QColor value);
+    void setDefaultBackgroundColor(const QColor &value);
 
     enum BackgroundStyle {
         RASTER_LAYER = 0,
@@ -485,6 +510,9 @@ public:
     bool trackTabletEventLatency(bool defaultValue = false) const;
     void setTrackTabletEventLatency(bool value);
 
+    bool ignoreHighFunctionKeys(bool defaultValue = false) const;
+    void setIgnoreHighFunctionKeys(bool value);
+
     bool testingAcceptCompressedTabletEvents(bool defaultValue = false) const;
     void setTestingAcceptCompressedTabletEvents(bool value);
 
@@ -519,6 +547,12 @@ public:
     bool compressKra(bool defaultValue = false) const;
     void setCompressKra(bool compress);
 
+    bool trimKra(bool defaultValue = false) const;
+    void setTrimKra(bool trim);
+
+    bool trimFramesImport(bool defaultValue = false) const;
+    void setTrimFramesImport(bool trim);
+
     bool toolOptionsInDocker(bool defaultValue = false) const;
     void setToolOptionsInDocker(bool inDocker);
 
@@ -534,21 +568,42 @@ public:
     bool kineticScrollingHiddenScrollbars(bool defaultValue = false) const;
     void setKineticScrollingHideScrollbars(bool scrollbar);
 
+    int zoomSteps(bool defaultValue = false) const;
+    void setZoomSteps(int steps);
+
+    int zoomMarginSize(bool defaultValue = false) const;
+    void setZoomMarginSize(int zoomMarginSize);
+
     void setEnableOpenGLFramerateLogging(bool value) const;
     bool enableOpenGLFramerateLogging(bool defaultValue = false) const;
 
     void setEnableBrushSpeedLogging(bool value) const;
     bool enableBrushSpeedLogging(bool defaultValue = false) const;
 
-    void setEnableAmdVectorizationWorkaround(bool value);
-    bool enableAmdVectorizationWorkaround(bool defaultValue = false) const;
+    void setDisableVectorOptimizations(bool value);
+    bool disableVectorOptimizations(bool defaultValue = false) const;
 
     void setDisableAVXOptimizations(bool value);
     bool disableAVXOptimizations(bool defaultValue = false) const;
 
+    void setAnimationPlaybackBackend(int value);
+    int animationPlaybackBackend(bool defaultValue = false) const;
+
     bool animationDropFrames(bool defaultValue = false) const;
     void setAnimationDropFrames(bool value);
 
+    bool autoPinLayersToTimeline(bool defaultValue = false) const;
+    void setAutoPinLayersToTimeline(bool value);
+
+    bool adaptivePlaybackRange(bool defaultValue = false) const;
+    void setAdaptivePlaybackRange(bool value);
+    
+    QString ffmpegLocation(bool defaultValue = false) const;
+    void setFFMpegLocation(const QString& value);
+
+    qreal timelineZoom(bool defaultValue = false) const;
+    void setTimelineZoom(qreal value);
+
     int scrubbingUpdatesDelay(bool defaultValue = false) const;
     void setScrubbingUpdatesDelay(int value);
 
@@ -572,6 +627,9 @@ public:
 
     bool showBrushHud(bool defaultValue = false) const;
     void setShowBrushHud(bool value);
+    
+    bool showPaletteBottomBar(bool defaultValue = false) const;
+    void setShowPaletteBottomBar(bool value);
 
     QString brushHudSetting(bool defaultValue = false) const;
     void setBrushHudSetting(const QString &value) const;
@@ -602,6 +660,38 @@ public:
     bool useZip64(bool defaultValue = false) const;
     void setUseZip64(bool value);
 
+    bool convertLayerColorSpaceInProperties(bool defaultValue = false) const;
+    void setConvertLayerColorSpaceInProperties(bool value);
+
+    bool renamePastedLayers(bool defaultValue = false) const;
+    void setRenamePastedLayers(bool value);
+
+    enum LayerInfoTextStyle {
+        INFOTEXT_NONE = 0,
+        INFOTEXT_SIMPLE,
+        INFOTEXT_BALANCED,
+        INFOTEXT_DETAILED
+    };
+    LayerInfoTextStyle layerInfoTextStyle(bool defaultValue = false) const;
+    void setLayerInfoTextStyle(LayerInfoTextStyle value);
+
+    int layerInfoTextOpacity(bool defaultValue = false) const;
+    void setLayerInfoTextOpacity(int value);
+
+    bool useInlineLayerInfoText(bool defaultValue = false) const;
+    void setUseInlineLayerInfoText(bool value);
+
+    bool useLayerSelectionCheckbox(bool defaultValue = false) const;
+    void setUseLayerSelectionCheckbox(bool value);
+
+    enum AssistantsDrawMode {
+        ASSISTANTS_DRAW_MODE_DIRECT = 0,             // no caching, draw directly on canvas
+        ASSISTANTS_DRAW_MODE_PIXMAP_CACHE = 1,
+        ASSISTANTS_DRAW_MODE_LARGE_PIXMAP_CACHE = 2,
+    };
+    AssistantsDrawMode assistantsDrawMode(bool defaultValue = false) const;
+    void setAssistantsDrawMode(AssistantsDrawMode value);
+
     template<class T>
     void writeEntry(const QString& name, const T& value) {
         m_cfg.writeEntry(name, value);
