@@ -1,24 +1,13 @@
 /*
- *  Copyright (c) 2011 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2011 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_tool_freehand_helper.h"
 
 #include <QTimer>
+#include <QElapsedTimer>
 #include <QQueue>
 
 #include <klocalizedstring.h>
@@ -44,6 +33,9 @@
 
 #include "strokes/freehand_stroke.h"
 #include "strokes/KisFreehandStrokeInfo.h"
+#include "KisAsynchronousStrokeUpdateHelper.h"
+#include "kis_canvas_resource_provider.h"
+#include <KisOptimizedBrushOutline.h>
 
 #include <math.h>
 
@@ -64,8 +56,10 @@ const qreal TIMING_UPDATE_INTERVAL = 50.0;
 
 struct KisToolFreehandHelper::Private
 {
+    KoCanvasResourceProvider *resourceManager;
     KisPaintingInformationBuilder *infoBuilder;
     KisStrokesFacade *strokesFacade;
+    KisAsynchronousStrokeUpdateHelper asyncUpdateHelper;
 
     KUndo2MagicString transactionText;
 
@@ -74,7 +68,7 @@ struct KisToolFreehandHelper::Private
 
     bool hasPaintAtLeastOnce;
 
-    QTime strokeTime;
+    QElapsedTimer strokeTime;
     QTimer strokeTimeoutTimer;
 
     QVector<KisFreehandStrokeInfo*> strokeInfos;
@@ -109,17 +103,17 @@ struct KisToolFreehandHelper::Private
     KisStabilizedEventsSampler stabilizedSampler;
     KisStabilizerDelayedPaintHelper stabilizerDelayedPaintHelper;
 
-    QTimer asynchronousUpdatesThresholdTimer;
-
     qreal effectiveSmoothnessDistance() const;
 };
 
 
 KisToolFreehandHelper::KisToolFreehandHelper(KisPaintingInformationBuilder *infoBuilder,
+                                             KoCanvasResourceProvider *resourceManager,
                                              const KUndo2MagicString &transactionText,
                                              KisSmoothingOptions *smoothingOptions)
     : m_d(new Private())
 {
+    m_d->resourceManager = resourceManager;
     m_d->infoBuilder = infoBuilder;
     m_d->transactionText = transactionText;
     m_d->smoothingOptions = KisSmoothingOptionsSP(
@@ -131,7 +125,6 @@ KisToolFreehandHelper::KisToolFreehandHelper(KisPaintingInformationBuilder *info
     m_d->strokeTimeoutTimer.setSingleShot(true);
     connect(&m_d->strokeTimeoutTimer, SIGNAL(timeout()), SLOT(finishStroke()));
     connect(&m_d->airbrushingTimer, SIGNAL(timeout()), SLOT(doAirbrushing()));
-    connect(&m_d->asynchronousUpdatesThresholdTimer, SIGNAL(timeout()), SLOT(doAsynchronousUpdate()));
     connect(&m_d->stabilizerPollTimer, SIGNAL(timeout()), SLOT(stabilizerPollAndPaint()));
     connect(m_d->smoothingOptions.data(), SIGNAL(sigSmoothingTypeChanged()), SLOT(slotSmoothingTypeChanged()));
 
@@ -141,7 +134,7 @@ KisToolFreehandHelper::KisToolFreehandHelper(KisPaintingInformationBuilder *info
                 });
     m_d->stabilizerDelayedPaintHelper.setUpdateOutlineCallback(
                 [this]() {
-                    emit requestExplicitUpdateOutline();
+                    Q_EMIT requestExplicitUpdateOutline();
                 });
 }
 
@@ -160,17 +153,18 @@ KisSmoothingOptionsSP KisToolFreehandHelper::smoothingOptions() const
     return m_d->smoothingOptions;
 }
 
-QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos,
-                                                   const KoPointerEvent *event,
-                                                   const KisPaintOpSettingsSP globalSettings,
-                                                   KisPaintOpSettings::OutlineMode mode) const
+KisOptimizedBrushOutline KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos,
+                                                               const KoPointerEvent *event,
+                                                               const KisPaintOpSettingsSP globalSettings,
+                                                               KisPaintOpSettings::OutlineMode mode) const
 {
     KisPaintOpSettingsSP settings = globalSettings;
-    KisPaintInformation info = m_d->infoBuilder->hover(savedCursorPos, event);
-    QPointF prevPoint = m_d->lastCursorPos.pushThroughHistory(savedCursorPos);
+    QPointF prevPoint = m_d->lastCursorPos.pushThroughHistory(savedCursorPos, currentZoom());
     qreal startAngle = KisAlgebra2D::directionBetweenPoints(prevPoint, savedCursorPos, 0);
     KisDistanceInformation distanceInfo(prevPoint, startAngle);
 
+    KisPaintInformation info;
+
     if (!m_d->strokeInfos.isEmpty()) {
         settings = m_d->resources->currentPaintOpPreset()->settings();
         if (m_d->stabilizerDelayedPaintHelper.running() &&
@@ -205,6 +199,8 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
         } else if (m_d->strokeInfos.first()->dragDistance->isStarted()) {
             distanceInfo = *m_d->strokeInfos.first()->dragDistance;
         }
+    } else {
+        info = m_d->infoBuilder->hover(savedCursorPos, event, !m_d->strokeInfos.isEmpty());
     }
 
     KisPaintInformation::DistanceInformationRegistrar registrar =
@@ -213,7 +209,7 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
     info.setRandomSource(m_d->fakeDabRandomSource);
     info.setPerStrokeRandomSource(m_d->fakeStrokeRandomSource);
 
-    QPainterPath outline = settings->brushOutline(info, mode);
+    KisOptimizedBrushOutline outline = settings->brushOutline(info, mode, currentPhysicalZoom());
 
     if (m_d->resources &&
         m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER &&
@@ -230,26 +226,25 @@ QPainterPath KisToolFreehandHelper::paintOpOutline(const QPointF &savedCursorPos
 
 void KisToolFreehandHelper::cursorMoved(const QPointF &cursorPos)
 {
-    m_d->lastCursorPos.pushThroughHistory(cursorPos);
+    m_d->lastCursorPos.pushThroughHistory(cursorPos, currentZoom());
 }
 
 void KisToolFreehandHelper::initPaint(KoPointerEvent *event,
                                       const QPointF &pixelCoords,
-                                      KoCanvasResourceProvider *resourceManager,
                                       KisImageWSP image, KisNodeSP currentNode,
                                       KisStrokesFacade *strokesFacade,
                                       KisNodeSP overrideNode,
                                       KisDefaultBoundsBaseSP bounds)
 {
-    QPointF prevPoint = m_d->lastCursorPos.pushThroughHistory(pixelCoords);
+    QPointF prevPoint = m_d->lastCursorPos.pushThroughHistory(pixelCoords, currentZoom());
     m_d->strokeTime.start();
     KisPaintInformation pi =
-        m_d->infoBuilder->startStroke(event, elapsedStrokeTime(), resourceManager);
+        m_d->infoBuilder->startStroke(event, elapsedStrokeTime(), m_d->resourceManager);
     qreal startAngle = KisAlgebra2D::directionBetweenPoints(prevPoint, pixelCoords, 0.0);
 
     initPaintImpl(startAngle,
                   pi,
-                  resourceManager,
+                  m_d->resourceManager,
                   image,
                   currentNode,
                   strokesFacade,
@@ -302,7 +297,11 @@ void KisToolFreehandHelper::initPaintImpl(qreal startAngle,
                    startDist);
 
     KisStrokeStrategy *stroke =
-        new FreehandStrokeStrategy(m_d->resources, m_d->strokeInfos, m_d->transactionText);
+        new FreehandStrokeStrategy(m_d->resources,
+                                   m_d->strokeInfos,
+                                   m_d->transactionText,
+                                   FreehandStrokeStrategy::SupportsContinuedInterstrokeData |
+                                   FreehandStrokeStrategy::SupportsTimedMergeId);
 
     m_d->strokeId = m_d->strokesFacade->startStroke(stroke);
 
@@ -313,8 +312,7 @@ void KisToolFreehandHelper::initPaintImpl(qreal startAngle,
         m_d->airbrushingTimer.setInterval(computeAirbrushTimerInterval());
         m_d->airbrushingTimer.start();
     } else if (m_d->resources->presetNeedsAsynchronousUpdates()) {
-        m_d->asynchronousUpdatesThresholdTimer.setInterval(80 /* msec */);
-        m_d->asynchronousUpdatesThresholdTimer.start();
+        m_d->asyncUpdateHelper.startUpdateStream(m_d->strokesFacade, m_d->strokeId);
     }
 
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER) {
@@ -329,6 +327,11 @@ void KisToolFreehandHelper::initPaintImpl(qreal startAngle,
     }
 }
 
+KoCanvasResourceProvider *KisToolFreehandHelper::resourceManager() const
+{
+    return m_d->resourceManager;
+}
+
 void KisToolFreehandHelper::paintBezierSegment(KisPaintInformation pi1, KisPaintInformation pi2,
                                                QPointF tangent1, QPointF tangent2)
 {
@@ -421,12 +424,18 @@ void KisToolFreehandHelper::paintBezierSegment(KisPaintInformation pi1, KisPaint
 
 qreal KisToolFreehandHelper::Private::effectiveSmoothnessDistance() const
 {
-    const qreal effectiveSmoothnessDistance =
-        !smoothingOptions->useScalableDistance() ?
-        smoothingOptions->smoothnessDistance() :
-        smoothingOptions->smoothnessDistance() / resources->effectiveZoom();
+    qreal zoomingCoeff = 1.0;
 
-    return effectiveSmoothnessDistance;
+    /// stabilizer has inverted meaning of the "scalable distance", because
+    /// it measures "samples", but not "distance"
+
+    if ((smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER) ^
+         smoothingOptions->useScalableDistance()) {
+
+        zoomingCoeff = 1.0 / resources->effectiveZoom();
+    }
+
+    return smoothingOptions->smoothnessDistance() * zoomingCoeff;
 }
 
 void KisToolFreehandHelper::paintEvent(KoPointerEvent *event)
@@ -445,7 +454,7 @@ void KisToolFreehandHelper::paint(KisPaintInformation &info)
      * Smooth the coordinates out using the history and the
      * distance. This is a heavily modified version of an algo used in
      * Gimp and described in https://bugs.kde.org/show_bug.cgi?id=281267 and
-     * http://www24.atwiki.jp/sigetch_2007/pages/17.html.  The main
+     * https://w.atwiki.jp/sigetch_2007/pages/17.html.  The main
      * differences are:
      *
      * 1) It uses 'distance' instead of 'velocity', since time
@@ -457,7 +466,7 @@ void KisToolFreehandHelper::paint(KisPaintInformation &info)
      * 3) 'Tail Aggressiveness' is used for controlling the end of the
      *    stroke
      *
-     * 4) The formila is a little bit different: 'Distance' parameter
+     * 4) The formula is a little bit different: 'Distance' parameter
      *    stands for $3 \Sigma$
      */
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::WEIGHTED_SMOOTHING
@@ -505,10 +514,10 @@ void KisToolFreehandHelper::paint(KisPaintInformation &info)
                 if (i < m_d->history.size() - 1) {
                     pressureGrad = nextInfo.pressure() - m_d->history.at(i + 1).pressure();
 
-                    const qreal tailAgressiveness = 40.0 * m_d->smoothingOptions->tailAggressiveness();
+                    const qreal tailAggressiveness = 40.0 * m_d->smoothingOptions->tailAggressiveness();
 
                     if (pressureGrad > 0.0 ) {
-                        pressureGrad *= tailAgressiveness * (1.0 - nextInfo.pressure());
+                        pressureGrad *= tailAggressiveness * (1.0 - nextInfo.pressure());
                         distance += pressureGrad * 3.0 * sigma; // (3 * sigma) --- holds > 90% of the region
                     }
                 }
@@ -615,14 +624,14 @@ void KisToolFreehandHelper::endPaint()
         m_d->airbrushingTimer.stop();
     }
 
-    if (m_d->asynchronousUpdatesThresholdTimer.isActive()) {
-        m_d->asynchronousUpdatesThresholdTimer.stop();
-    }
-
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::STABILIZER) {
         stabilizerEnd();
     }
 
+    if (m_d->asyncUpdateHelper.isActive()) {
+        m_d->asyncUpdateHelper.endUpdateStream();
+    }
+
     /**
      * There might be some timer events still pending, so
      * we should cancel them. Use this flag for the purpose.
@@ -632,10 +641,12 @@ void KisToolFreehandHelper::endPaint()
     m_d->strokeInfos.clear();
 
     // last update to complete rendering if there is still something pending
-    doAsynchronousUpdate(true);
+    m_d->strokesFacade->addJob(m_d->strokeId,
+       new KisAsynchronousStrokeUpdateHelper::UpdateData(true));
 
     m_d->strokesFacade->endStroke(m_d->strokeId);
     m_d->strokeId.clear();
+    m_d->infoBuilder->reset();
 }
 
 void KisToolFreehandHelper::cancelPaint()
@@ -648,8 +659,8 @@ void KisToolFreehandHelper::cancelPaint()
         m_d->airbrushingTimer.stop();
     }
 
-    if (m_d->asynchronousUpdatesThresholdTimer.isActive()) {
-        m_d->asynchronousUpdatesThresholdTimer.stop();
+    if (m_d->asyncUpdateHelper.isActive()) {
+        m_d->asyncUpdateHelper.cancelUpdateStream();
     }
 
     if (m_d->stabilizerPollTimer.isActive()) {
@@ -715,6 +726,10 @@ KisToolFreehandHelper::getStabilizedPaintInfo(const QQueue<KisPaintInformation>
                                elapsedStrokeTime(),
                                lastPaintInfo.drawingSpeed());
 
+    result.setCanvasRotation(lastPaintInfo.canvasRotation());
+    result.setCanvasMirroredH(lastPaintInfo.canvasMirroredH());
+    result.setCanvasMirroredV(lastPaintInfo.canvasMirroredV());
+
     if (queue.size() > 1) {
         QQueue<KisPaintInformation>::const_iterator it = queue.constBegin();
         QQueue<KisPaintInformation>::const_iterator end = queue.constEnd();
@@ -804,7 +819,7 @@ void KisToolFreehandHelper::stabilizerPollAndPaint()
     if (m_d->stabilizerDelayedPaintHelper.running()) {
         m_d->stabilizerDelayedPaintHelper.update(delayedPaintTodoItems);
     } else {
-        emit requestExplicitUpdateOutline();
+        Q_EMIT requestExplicitUpdateOutline();
     }
 }
 
@@ -877,22 +892,29 @@ void KisToolFreehandHelper::doAirbrushing()
                                       prevPaint.perspective(),
                                       elapsedStrokeTime(),
                                       0.0);
+        nextPaint.setCanvasRotation(prevPaint.canvasRotation());
+        nextPaint.setCanvasMirroredH(prevPaint.canvasMirroredH());
+        nextPaint.setCanvasMirroredV(prevPaint.canvasMirroredV());
         paint(nextPaint);
     }
 }
 
-void KisToolFreehandHelper::doAsynchronousUpdate(bool forceUpdate)
-{
-    m_d->strokesFacade->addJob(m_d->strokeId,
-                               new FreehandStrokeStrategy::UpdateData(forceUpdate));
-}
-
 int KisToolFreehandHelper::computeAirbrushTimerInterval() const
 {
     qreal realInterval = m_d->resources->airbrushingInterval() * AIRBRUSH_INTERVAL_FACTOR;
     return qMax(1, qFloor(realInterval));
 }
 
+qreal KisToolFreehandHelper::currentZoom() const
+{
+    return m_d->resourceManager ? m_d->resourceManager->resource(KoCanvasResource::EffectiveZoom).toReal() : 1.0;
+}
+
+qreal KisToolFreehandHelper::currentPhysicalZoom() const
+{
+    return m_d->resourceManager ? m_d->resourceManager->resource(KoCanvasResource::EffectivePhysicalZoom).toReal() : 1.0;
+}
+
 void KisToolFreehandHelper::paintAt(int strokeInfoId,
                                     const KisPaintInformation &pi)
 {
