@@ -1,22 +1,10 @@
 /*
- *  Copyright (C) 2007 Cyrille Berger <cberger@cberger.net>
- *  Copyright (C) 2011 Boudewijn Rempt <boud@valdyas.org>
- *  Copyright (C) 2011 Srikanth Tiyyagura <srikanth.tulasiram@gmail.com>
- *  Copyright (C) 2015 Wolthera van Hövell tot Westerflier <griffinvalley@gmail.com>
+ *  SPDX-FileCopyrightText: 2007 Cyrille Berger <cberger@cberger.net>
+ *  SPDX-FileCopyrightText: 2011 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2011 Srikanth Tiyyagura <srikanth.tulasiram@gmail.com>
+ *  SPDX-FileCopyrightText: 2015 Wolthera van Hövell tot Westerflier <griffinvalley@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_advanced_color_space_selector.h"
@@ -44,6 +32,8 @@
 
 #include <kis_debug.h>
 
+#include <kstandardguiitem.h>
+
 struct KisAdvancedColorSpaceSelector::Private {
     Ui_WdgColorSpaceSelectorAdvanced* colorSpaceSelector;
     QString knsrcFile;
@@ -58,19 +48,31 @@ KisAdvancedColorSpaceSelector::KisAdvancedColorSpaceSelector(QWidget* parent, co
     d->colorSpaceSelector = new Ui_WdgColorSpaceSelectorAdvanced;
     d->colorSpaceSelector->setupUi(this);
 
+    {
+        QSizePolicy policy = d->colorSpaceSelector->TongueWidget->sizePolicy();
+        policy.setHeightForWidth(true);
+        d->colorSpaceSelector->TongueWidget->setSizePolicy(policy);
+    }
+
+    {
+        QSizePolicy policy = d->colorSpaceSelector->TRCwidget->sizePolicy();
+        policy.setHeightForWidth(true);
+        d->colorSpaceSelector->TRCwidget->setSizePolicy(policy);
+    }
+
     d->colorSpaceSelector->cmbColorModels->setIDList(KoColorSpaceRegistry::instance()->colorModelsList(KoColorSpaceRegistry::OnlyUserVisible));
     fillCmbDepths(d->colorSpaceSelector->cmbColorModels->currentItem());
 
-    d->colorSpaceSelector->bnInstallProfile->setIcon(KisIconUtils::loadIcon("document-open"));
-    d->colorSpaceSelector->bnInstallProfile->setToolTip( i18n("Open Color Profile") );
+    d->colorSpaceSelector->bnInstallProfile->setIcon(koIcon("document-import-16"));
+    d->colorSpaceSelector->bnInstallProfile->setText(i18n("Import profile"));
 
-    connect(d->colorSpaceSelector->cmbColorModels, SIGNAL(activated(const KoID &)),
-            this, SLOT(fillCmbDepths(const KoID &)));
-    connect(d->colorSpaceSelector->cmbColorDepth, SIGNAL(activated(const KoID &)),
+    connect(d->colorSpaceSelector->cmbColorModels, SIGNAL(activated(KoID)),
+            this, SLOT(fillCmbDepths(KoID)));
+    connect(d->colorSpaceSelector->cmbColorDepth, SIGNAL(activated(KoID)),
             this, SLOT(fillLstProfiles()));
-    connect(d->colorSpaceSelector->cmbColorModels, SIGNAL(activated(const KoID &)),
+    connect(d->colorSpaceSelector->cmbColorModels, SIGNAL(activated(KoID)),
             this, SLOT(fillLstProfiles()));
-    connect(d->colorSpaceSelector->lstProfile, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)),
+    connect(d->colorSpaceSelector->lstProfile, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
             this, SLOT(colorSpaceChanged()));
     connect(this, SIGNAL(selectionChanged(bool)),
             this, SLOT(fillDescription()));
@@ -79,6 +81,8 @@ KisAdvancedColorSpaceSelector::KisAdvancedColorSpaceSelector(QWidget* parent, co
 
     connect(d->colorSpaceSelector->bnInstallProfile, SIGNAL(clicked()), this, SLOT(installProfile()));
 
+    KGuiItem::assign(d->colorSpaceSelector->bnOK->button(QDialogButtonBox::Ok), KStandardGuiItem::ok());
+    KGuiItem::assign(d->colorSpaceSelector->bnOK->button(QDialogButtonBox::Cancel), KStandardGuiItem::cancel());
     connect(d->colorSpaceSelector->bnOK, SIGNAL(accepted()), this, SLOT(accept()));
     connect(d->colorSpaceSelector->bnOK, SIGNAL(rejected()), this, SLOT(reject()));
 
@@ -201,6 +205,9 @@ void KisAdvancedColorSpaceSelector::fillDescription()
             d->colorSpaceSelector->TongueWidget->setRGBData(whitepoint, colorants);
         }
         d->colorSpaceSelector->TongueWidget->setGamut(currentColorSpace()->gamutXYY());
+        TransferCharacteristics detCharacteristics = currentColorSpace()->profile()->getTransferCharacteristics();
+        QString detCharacteristicsStr =
+            currentColorSpace()->profile()->getTransferCharacteristicName(detCharacteristics);
         estimatedTRC = currentColorSpace()->profile()->getEstimatedTRC();
         QString estimatedCurve = " Estimated curve: ";
         QPolygonF redcurve;
@@ -222,22 +229,32 @@ void KisAdvancedColorSpaceSelector::fillDescription()
             d->colorSpaceSelector->TRCwidget->setRGBCurve(redcurve, greencurve, bluecurve);
         } else {
             QPolygonF curve = currentColorSpace()->estimatedTRCXYY();
-            redcurve << curve.at(0) << curve.at(1) << curve.at(2) << curve.at(3) << curve.at(4);
-            greencurve << curve.at(5) << curve.at(6) << curve.at(7) << curve.at(8) << curve.at(9);
-            bluecurve << curve.at(10) << curve.at(11) << curve.at(12) << curve.at(13) << curve.at(14);
+            int numPoints = curve.size() / 3;
+            for (int i = 0; i < numPoints; i++) {
+                redcurve << curve.at(i);
+                greencurve << curve.at(i + numPoints);
+                bluecurve << curve.at(i + 2*numPoints);
+            }
             d->colorSpaceSelector->TRCwidget->setRGBCurve(redcurve, greencurve, bluecurve);
         }
 
-        if (estimatedTRC[0] == -1) {
-            d->colorSpaceSelector->TRCwidget->setToolTip("<html><head/><body>"+whatissRGB+"<br />"+estimatedCurve+"</body></html>");
+        if (estimatedTRC[0] != -1 && currentColorSpace()->profile()->hasTRC()) {
+            d->colorSpaceSelector->TRCwidget->setToolTip(
+                "<html><head/><body>" + estimatedGamma + detCharacteristicsStr + "<br/>"
+                + QString::number(estimatedTRC[0]) + "," + QString::number(estimatedTRC[1]) + ","
+                + QString::number(estimatedTRC[2]) + "<br/>" + estimatedCurve + "</body></html>");
         } else {
-            d->colorSpaceSelector->TRCwidget->setToolTip("<html><head/><body>"+estimatedGamma + QString::number(estimatedTRC[0]) + "," + QString::number(estimatedTRC[1]) + "," + QString::number(estimatedTRC[2])+"<br />"+estimatedCurve+"</body></html>");
+            d->colorSpaceSelector->TRCwidget->setToolTip("<html><head/><body>" + estimatedGamma + detCharacteristicsStr
+                                                         + "<br/>" + estimatedCurve + "</body></html>");
         }
     }
     else if (currentModelStr == "GRAYA") {
         QVector <qreal> whitepoint = currentColorSpace()->profile()->getWhitePointxyY();
         d->colorSpaceSelector->TongueWidget->setGrayData(whitepoint);
         d->colorSpaceSelector->TongueWidget->setGamut(currentColorSpace()->gamutXYY());
+        TransferCharacteristics detCharacteristics = currentColorSpace()->profile()->getTransferCharacteristics();
+        QString detCharacteristicsStr =
+            currentColorSpace()->profile()->getTransferCharacteristicName(detCharacteristics);
         estimatedTRC = currentColorSpace()->profile()->getEstimatedTRC();
         QString estimatedCurve = " Estimated curve: ";
         QPolygonF tonecurve;
@@ -254,10 +271,13 @@ void KisAdvancedColorSpaceSelector::fillDescription()
             d->colorSpaceSelector->TRCwidget->setProfileDataAvailable(false);
         }
         d->colorSpaceSelector->TRCwidget->setGreyscaleCurve(tonecurve);
-        if (estimatedTRC[0] == -1) {
-            d->colorSpaceSelector->TRCwidget->setToolTip("<html><head/><body>"+whatissRGB+"<br />"+estimatedCurve+"</body></html>");
+        if (estimatedTRC[0] != -1 && currentColorSpace()->profile()->hasTRC()) {
+            d->colorSpaceSelector->TRCwidget->setToolTip("<html><head/><body>" + estimatedGamma + detCharacteristicsStr
+                                                         + "<br/>" + QString::number(estimatedTRC[0]) + "<br/>"
+                                                         + estimatedCurve + "</body></html>");
         } else {
-            d->colorSpaceSelector->TRCwidget->setToolTip("<html><head/><body>"+estimatedGamma + QString::number(estimatedTRC[0])+"<br />"+estimatedCurve+"</body></html>");
+            d->colorSpaceSelector->TRCwidget->setToolTip("<html><head/><body>" + estimatedGamma + detCharacteristicsStr
+                                                         + "<br/>" + estimatedCurve + "</body></html>");
         }
     }
     else if (currentModelStr == "CMYKA") {
@@ -281,10 +301,13 @@ void KisAdvancedColorSpaceSelector::fillDescription()
             d->colorSpaceSelector->TRCwidget->setGreyscaleCurve(tonecurve);
         } else {
             QPolygonF curve = currentColorSpace()->estimatedTRCXYY();
-            cyancurve << curve.at(0) << curve.at(1) << curve.at(2) << curve.at(3) << curve.at(4);
-            magentacurve << curve.at(5) << curve.at(6) << curve.at(7) << curve.at(8) << curve.at(9);
-            yellowcurve << curve.at(10) << curve.at(11) << curve.at(12) << curve.at(13) << curve.at(14);
-            tonecurve << curve.at(15) << curve.at(16) << curve.at(17) << curve.at(18) << curve.at(19);
+            int numPoints = curve.size() / 4;
+            for (int i = 0; i < numPoints; i++) {
+                cyancurve << curve.at(i);
+                magentacurve << curve.at(i + numPoints);
+                yellowcurve << curve.at(i + 2*numPoints);
+                tonecurve << curve.at(i + 3*numPoints);
+            }
             d->colorSpaceSelector->TRCwidget->setCMYKCurve(cyancurve, magentacurve, yellowcurve, tonecurve);
         }
         d->colorSpaceSelector->TRCwidget->setToolTip(i18nc("@info:tooltip","Estimated Gamma cannot be retrieved for CMYK."));
@@ -352,17 +375,26 @@ void KisAdvancedColorSpaceSelector::fillDescription()
             d->colorSpaceSelector->TRCwidget->setProfileDataAvailable(false);
         }
         d->colorSpaceSelector->TongueWidget->setGamut(currentColorSpace()->gamutXYY());
-        d->colorSpaceSelector->TRCwidget->setToolTip(i18nc("@info:tooltip","Estimated Gamma cannot be retrieved for YCrCb."));
+        d->colorSpaceSelector->TRCwidget->setToolTip(i18nc("@info:tooltip","Estimated Gamma cannot be retrieved for YCbCr."));
     }
 
     d->colorSpaceSelector->textProfileDescription->clear();
-    if (profileList.isEmpty()==false) {
-        d->colorSpaceSelector->textProfileDescription->append("<h3>"+i18nc("About <Profilename>","About ")  +  currentColorSpace()->name()  +  "/"  +  profileName  +  "</h3>");
-        d->colorSpaceSelector->textProfileDescription->append("<p>"+ i18nc("ICC profile version","ICC Version: ")  + QString::number(currentColorSpace()->profile()->version())  +  "</p>");
-        //d->colorSpaceSelector->textProfileDescription->append("<p>"+ i18nc("Who made the profile?","Manufacturer: ")  + currentColorSpace()->profile()->manufacturer()  +  "</p>"); //This would work if people actually wrote the manufacturer into the manufacturer fiedl...
-        d->colorSpaceSelector->textProfileDescription->append("<p>"+ i18nc("What is the copyright? These are from embedded strings from the icc profile, so they default to english.","Copyright: ")  + currentColorSpace()->profile()->copyright()  +  "</p>");
+    if (!profileList.isEmpty()) {
+        d->colorSpaceSelector->textProfileDescription->append(
+            QString("<h3>%1</h3>").arg(i18nc("About <Profilename>", "About %1/%2", currentColorSpace()->name().toHtmlEscaped(), profileName.toHtmlEscaped())));
+        d->colorSpaceSelector->textProfileDescription->append(
+            QString("<p>%1</p>").arg(i18nc("ICC profile version", "ICC Version: %1", QString::number(currentColorSpace()->profile()->version()))));
+        if (currentColorSpace()->profile()->manufacturer() != profileName)
+            d->colorSpaceSelector->textProfileDescription->append(
+                QString("<p>%1</p>").arg(i18nc("Who made the profile?", "Manufacturer: %1", currentColorSpace()->profile()->manufacturer())));
+        d->colorSpaceSelector->textProfileDescription->append(
+            QString("<p>%1</p>")
+                .arg(i18nc("What is the copyright? These are from embedded strings from the icc profile, so they "
+                           "default to english.",
+                           "Copyright: %1",
+                           currentColorSpace()->profile()->copyright().toHtmlEscaped())));
     } else {
-        d->colorSpaceSelector->textProfileDescription->append("<h3>" + profileName  +  "</h3>");
+        d->colorSpaceSelector->textProfileDescription->append(QString("<h3>%1</h3>").arg(profileName.toHtmlEscaped()));
     }
 
     if (currentModelStr == "RGBA") {
@@ -392,11 +424,11 @@ void KisAdvancedColorSpaceSelector::fillDescription()
                                                                     "<b><a href=\"https://en.wikipedia.org/wiki/Grayscale\">Grayscale</a></b> only allows for "
                                                                     "gray values and transparent values. Grayscale images use half "
                                                                     "the memory and disk space compared to an RGB image of the same bit-depth.<br/>"
-                                                                    "Grayscale is useful for inking and greyscale images. In "
+                                                                    "Grayscale is useful for inking and grayscale images. In "
                                                                     "Krita, you can mix Grayscale and RGB layers in the same image.")+"</p>");
     } else if (currentModelStr == "LABA") {
         d->colorSpaceSelector->textProfileDescription->append("<p>"+i18nc("If the selected model is LAB",
-                                                                    "<b><a href=\"https://en.wikipedia.org/wiki/Lab_color_space\">L*a*b</a></b>. <b>L<b> stands for Lightness, "
+                                                                    "<b><a href=\"https://en.wikipedia.org/wiki/Lab_color_space\">L*a*b</a></b>. <b>L</b> stands for Lightness, "
                                                                     "the <b>a</b> and <b>b</b> components represent color channels.<br/>"
                                                                     "L*a*b is a special model for color correction. It is based on human perception, meaning that it "
                                                                     "tries to encode the difference in lightness, red-green balance and yellow-blue balance. "
@@ -411,17 +443,15 @@ void KisAdvancedColorSpaceSelector::fillDescription()
                                                                     "model designed for video encoding. It is based on human perception, meaning that it tries to "
                                                                     "encode the difference in lightness, red-green balance and yellow-blue balance. Chroma in "
                                                                     "this case is then a word indicating a special type of saturation, in these cases the saturation "
-                                                                    "of Red and Blue, of which the desaturated equivalents are Green and Yellow respectively. It "
-                                                                    "is available to open up certain images correctly, but Krita does not currently ship a profile for "
-                                                                    "this due to lack of open source ICC profiles for YCrCb.")+"</p>");
+                                                                    "of Red and Blue, of which the desaturated equivalents are Green and Yellow respectively.")+"</p>");
     }
 
     QString currentDepthStr = d->colorSpaceSelector->cmbColorDepth->currentItem().id();
 
     if (currentDepthStr == "U8") {
         d->colorSpaceSelector->textProfileDescription->append("<p>"+i18nc("When the selected Bitdepth is 8",
-                                                                    "<b>8 bit integer</b>: The default amount of colors per channel. Each channel will have 256 values available, "
-                                                                    "leading to a total amount of 256*amount of channels. Recommended to use for images intended for the web, "
+                                                                    "<b>8 bit integer</b>: The default number of colors per channel. Each channel will have 256 values available, "
+                                                                    "leading to a total amount of colors of 256 to the power of the number of channels. Recommended to use for images intended for the web, "
                                                                     "or otherwise simple images.")+"</p>");
     }
     else if (currentDepthStr == "U16") {
@@ -472,7 +502,7 @@ void KisAdvancedColorSpaceSelector::fillDescription()
             possibleConversionIntents += "<li>"+i18n("Absolute Colorimetric")+"</li>";
         }
         if (currentColorSpace()->profile()->supportsSaturation()){
-            possibleConversionIntents += "<li>"+i18n("Saturation")+"</li>";
+            possibleConversionIntents += "<li>"+i18nc("ICC profile rendering intent", "Saturation")+"</li>";
         }
         possibleConversionIntents += "</ul></ul></p>";
         d->colorSpaceSelector->textProfileDescription->append(possibleConversionIntents);
@@ -482,8 +512,8 @@ void KisAdvancedColorSpaceSelector::fillDescription()
         d->colorSpaceSelector->textProfileDescription->append("<p>"+i18nc("These are Elle Stone's notes on her profiles that we ship.",
                                                                     "<p><b>Extra notes on profiles by Elle Stone:</b></p>"
                                                                     "<p><i>Krita comes with a number of high quality profiles created by "
-                                                                    "<a href=\"http://ninedegreesbelow.com\">Elle Stone</a>. This is a summary. Please check "
-                                                                    "<a href=\"http://ninedegreesbelow.com/photography/lcms-make-icc-profiles.html\">the full documentation</a> as well.</i></p>"));
+                                                                    "<a href=\"https://ninedegreesbelow.com\">Elle Stone</a>. This is a summary. Please check "
+                                                                    "<a href=\"https://ninedegreesbelow.com/photography/lcms-make-icc-profiles.html\">the full documentation</a> as well.</i></p>"));
 
                 if (profileName.contains("ACES-")) {
 
@@ -522,7 +552,7 @@ void KisAdvancedColorSpaceSelector::fillDescription()
                                                                         "actually has a slightly larger color gamut (to capture some fringe colors that barely qualify "
                                                                         "as real when viewed by the standard observer) and uses the D50 white point.</p><p>"
                                                                         "Just like the ACES color space, AllColorsRGB holds a high percentage of imaginary colors. See the Completely "
-                                                                        "<a href=\"http://ninedegreesbelow.com/photography/xyz-rgb.html\">"
+                                                                        "<a href=\"https://ninedegreesbelow.com/photography/xyz-rgb.html\">"
                                                                         "Painless Programmer's Guide to XYZ, RGB, ICC, xyY, and TRCs</a> for more information about imaginary "
                                                                         "colors.</p><p>"
                                                                         "There is no particular reason why anyone would want to use this profile "
@@ -559,12 +589,14 @@ void KisAdvancedColorSpaceSelector::fillDescription()
                                                                         "gamut to include all printable and most real world colors. It includes some imaginary colors "
                                                                         "and excludes some of the real world blues and violet blues that can be captured by digital "
                                                                         "cameras. It also excludes some very saturated 'camera-captured' yellows as interpreted by "
-                                                                        "some (and probably many) camera matrix input profiles.<p>"
+                                                                        "some (and probably many) camera matrix input profiles.</p><p>"
                                                                         "The ProPhotoRGB primaries are "
                                                                         "hard-coded into Adobe products such as Lightroom and the Dng-DCP camera 'profiles'. However, "
                                                                         "other than being large enough to hold a lot of colors, ProPhotoRGB has no particular merit "
-                                                                        "as an RGB working space. Personally and for most editing purposes, I recommend BetaRGB, Rec2020, "
-                                                                        "or the ACEScg profiles ProPhotoRGB.</p>"));
+                                                                        "as an RGB working space. Personally I recommend the Rec.2020 or ACEScg profiles over "
+                                                                        "ProPhotoRGB. But if you have an already well-established workflow using ProPhotoRGB, you "
+                                                                        "might find a shift to another RGB working space a little odd, at least at first, and so you "
+                                                                        "have to weight the pros and cons of changing your workflow.</p>"));
         }
         if (profileName.contains("Rec2020-")) {
 
@@ -621,9 +653,9 @@ void KisAdvancedColorSpaceSelector::fillDescription()
                                                                         "should be done on perceptually uniform RGB. Make sure you use the V4 versions for editing high bit depth "
                                                                         "images.</p>"));
         }
-        if (profileName.contains("-srgbtrc") || profileName.contains("-g22") || profileName.contains("-g18") || profileName.contains("-bt709")) {
+        if (profileName.contains("-srgbtrc") || profileName.contains("-g22") || profileName.contains("-g18") || profileName.contains("-rec709")) {
             d->colorSpaceSelector->textProfileDescription->append(i18nc("From Elle's notes.",
-                                                                        "<p>The profiles that end in '-srgbtrc.icc', '-g22.icc', and '-bt709.icc' have approximately but not exactly "
+                                                                        "<p>The profiles that end in '-srgbtrc.icc', '-g22.icc', and '-rec709.icc' have approximately but not exactly "
                                                                         "perceptually uniform TRCs. ProPhotoRGB's gamma=1.8 TRC is not quite as close to being perceptually uniform.</p>"));
         }
         if (d->colorSpaceSelector->cmbColorDepth->currentItem().id()=="U8") {
@@ -660,7 +692,7 @@ QString KisAdvancedColorSpaceSelector::nameWhitePoint(QVector <double> whitePoin
         return name;
     }
     //B   (0.34980, 0.35270) (4874K) (Direct Sunlight at noon)(obsolete)
-    //C   (0.31039, 0.31905) (6774K) (avarage/north sky daylight)(obsolete)
+    //C   (0.31039, 0.31905) (6774K) (average/north sky daylight)(obsolete)
     //D50 (0.34773, 0.35952) (5003K) (Horizon Light, default color of white paper, ICC profile standard illuminant)
     if ((whitePoint[0]>0.34773-0.005 && whitePoint[0]<0.34773 + 0.005) &&
             (whitePoint[1]>0.35952-0.005 && whitePoint[1]<0.35952 + 0.005)){
@@ -762,7 +794,7 @@ void KisAdvancedColorSpaceSelector::colorSpaceChanged()
     bool valid = d->colorSpaceSelector->lstProfile->count() != 0;
     emit(selectionChanged(valid));
     if (valid) {
-        emit colorSpaceChanged(currentColorSpace());
+        Q_EMIT colorSpaceChanged(currentColorSpace());
     }
 }
 
