@@ -1,20 +1,7 @@
 /* This file is part of the KDE project
-   Copyright (C) 2013 - 2014 Yue Liu <yue.liu@mail.com>
+   SPDX-FileCopyrightText: 2013-2014 Yue Liu <yue.liu@mail.com>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KOFILEDIALOG_H
@@ -65,12 +52,6 @@ public:
 
     void setCaption(const QString &caption);
 
-
-    /**
-     * Use the given filename as a proposal.
-     */
-    void setProposedFileName(const QString &filename);
-
     /**
      * @brief setDefaultDir set the default directory to defaultDir.
      *
@@ -78,6 +59,14 @@ public:
      */
     void setDefaultDir(const QString &defaultDir, bool force = false);
 
+    /**
+     * @brief setDirectoryUrl set the default URI to defaultUri.
+     * @param defaultUri a Uri to a file from some ContentProvider
+     *
+     * Used only on Android.
+     */
+    void setDirectoryUrl(const QUrl &defaultUri);
+
     /**
      * @brief setImageFilters sets the name filters for the file dialog to all
      * image formats Qt's QImageReader supports.
@@ -107,6 +96,10 @@ public:
 
     QString selectedMimeType() const;
 
+public Q_SLOTS:
+    // Set default file extension matching the filter.
+    void onFilterSelected(const QString &filter);
+
 private:
     /**
      * @brief splitNameFilter take a single line of a QDialog name filter and split it
@@ -125,11 +118,10 @@ private:
     QString getUsedDir(const QString &dialogName);
     void saveUsedDir(const QString &fileName, const QString &dialogName);
 
-    const QStringList getFilterStringListFromMime(const QStringList &mimeList,
+    // Get list of user friendly descriptions and map of these descriptions to the proposed file extension.
+    const QPair<QStringList, QMap<QString, QString>> getFilterStringListFromMime(const QStringList &mimeList,
                                                   bool withAllSupportedEntry = false);
 
-
-
     class Private;
     Private * const d;
 };
